//
//  SmuleGlobe.frag
//  ExplorerGlobe
//
//  Created by Anton Holmberg on 3/20/16.
//  Copyright © 2016 Smule. All rights reserved.
//

#ifdef GL_ES
precision highp float;
#else
// define qualifiers to empty so code can compile without deleting qualifiers
#define highp
#define mediump
#define lowp
#endif

#define PI 3.1415926538

uniform sampler2D u_earthTex;
uniform vec3 u_cameraPositionWS;

uniform vec3 u_planetRimColor;
uniform float u_planetRimFalloff;

uniform vec3 u_continentColor;
uniform vec3 u_continentRimColor;
uniform float u_continentRimColorAmount;

uniform vec3 u_oceanColor;
uniform vec3 u_oceanHighlightColor;
uniform float u_oceanHighlightAmount;

uniform vec4 u_pulseColor;
uniform float u_pulseColorFalloff;
uniform float u_pulseDisplacementAmount;
uniform float u_pulseDisplacementFalloff;
uniform float u_pulseTextureOffsetAmount;
uniform float u_pulseYPositionWS;

// NOTE: THIS SHADER HAS ALL OF THE MINIMUM UNIFORM SLOTS FILLED UP.
// The OpenGL ES specification specifies that implementation must support
// a minimum of 16 uniforms in shaders. Above this limit will probably work
// on most devices, but we should avoid going over this limit just to be
// safe. If you want to add more uniforms, you'll need to consolidate some
// of the existing uniforms into vectors (intead of single floats)

// NOTE: see note in vertex shader about these varyings
varying vec3 v_positionMS; // model space
varying vec3 v_positionWS; // world space
varying vec3 v_normalWS; // world space
varying vec2 v_texCoord;


float fresnel(vec3 viewDirWS, vec3 normalWS, float power, float bias)
{
    float cos0 = dot(viewDirWS, v_normalWS) - bias;
    return pow(1.0 - cos0, power);
}

float fresnel(vec3 viewDirWS, vec3 normalWS, float power)
{
    return fresnel(viewDirWS, normalWS, power, 0.0);
}


float calculatePulseFactor()
{
    // NOTE: keep this method in sync with the equivalent method in
    // vertex shader AND in SmuleGlobe.cpp
    float pulseFactor = 1.0 - abs(u_pulseYPositionWS - v_positionMS.y);
    pulseFactor = clamp(pulseFactor, 0.0, 1.0);
    pulseFactor = (cos(-PI + pulseFactor * PI) + 1.0) * 0.5;
    return pulseFactor;
}


void main()
{
    vec3 viewDirWS = normalize(v_positionWS - u_cameraPositionWS);
    
    float pulseFactor = calculatePulseFactor();
    float pulseTextureOffsetFactor = pow(pulseFactor, u_pulseDisplacementFalloff);

    // read mask values from texture. The left half of globe is in R channel, and the right
    // half of globe is in G channel. So we have to shift x coordinate by 0.5 once we
    // reach half-way, and then multiply x coordinate by 2 to read proper values. Then
    // we need to mask off the appropriate channel, and add mask values together to get final mask
    vec2 displacedTexCoord = v_texCoord + vec2(0, u_pulseTextureOffsetAmount * pulseTextureOffsetFactor);
    vec2 channelMask = vec2(1.0, 0.0);
    if (displacedTexCoord.x > 0.5) {
        displacedTexCoord.x -= 0.5;
        channelMask = vec2(0.0, 1.0);
    }
    displacedTexCoord.x *= 2.0;
    vec2 maskData = texture2D(u_earthTex, displacedTexCoord).xy * channelMask;
    float continentMask = maskData.x + maskData.y;
    float oceanMask = 1.0 - continentMask;
    
    // calculate continent color
    vec3 continentColor = u_continentColor * continentMask;
    vec3 continentRimColor = u_continentRimColor * continentMask;
    continentRimColor *= fresnel(viewDirWS, v_normalWS, u_continentRimColorAmount);
    continentColor += continentRimColor;
    
    // calculate ocean color
    vec3 oceanColor = u_oceanColor * oceanMask;
    vec3 oceanHighlightColor = u_oceanHighlightColor * oceanMask;
    float oceanHighlightAmount = fresnel(viewDirWS, v_normalWS, u_oceanHighlightAmount);
    oceanHighlightAmount *= oceanHighlightAmount;
    oceanHighlightAmount = 1.0 - oceanHighlightAmount;
    oceanHighlightColor *= oceanHighlightAmount * 0.5;
    oceanColor += oceanHighlightColor;
    
    // add continent and ocean together to get full planet color
    vec3 planetColor = continentColor + oceanColor;
    
    // apply spotlight effect to the globe by multiplying in a darker tint
    // and a lighter tint using an inverted fresnel
//    float spotlightAmount = fresnel(viewDirWS, v_normalWS, 3.0, 0.1);
//    spotlightAmount = (1.0 - spotlightAmount);
//    vec3 darkTintColor = vec3(1.0, 1.0, 1.0);
//    vec3 lightTintColor = vec3(1.0, 1.0, 1.0);
//    vec3 darkerPlanetColor = planetColor * darkTintColor * 0.55;
//    vec3 lighterPlanetColor = planetColor * lightTintColor * 0.75;
//    planetColor = mix(darkerPlanetColor, lighterPlanetColor, spotlightAmount);
    
    // add rim around planet
    float planetRimAmount = fresnel(viewDirWS, v_normalWS, u_planetRimFalloff, 0.1);
    vec3 planetRimColor = u_planetRimColor * planetRimAmount;
    planetColor = mix(planetColor, u_planetRimColor, planetRimAmount);
    
    // add pulse to planetColor
    float pulseColorFactor = pow(pulseFactor, u_pulseColorFalloff);
    planetColor += u_pulseColor.xyz * u_pulseColor.w * pulseColorFactor;
    
    gl_FragColor = vec4(planetColor, 1);
}
